@@ -1,7 +1,7 @@
 /* version.c: The opieversion() library function.
 
 %%% portions-copyright-cmetz-96
-Portions of this software are Copyright 1996-1998 by Craig Metz, All Rights
+Portions of this software are Copyright 1996-1999 by Craig Metz, All Rights
 Reserved. The Inner Net License Version 2 applies to these portions of
 the software.
 You should have received a copy of the license with this software. If
@@ -17,9 +17,13 @@ License Agreement applies to this software.
 	Modified by cmetz for OPIE 2.2. Use FUNCTION declaration et al.
         Created at NRL for OPIE 2.2 from opiesubr.c.
 */
+#include <stdio.h>
+#include <stdlib.h>
 #include "opie_cfg.h"
 #include "opie.h"
 
+#include <stdio.h>
+
 VOIDRET opieversion FUNCTION_NOARGS
 {
   printf("\nOPIE %s (%s)\n\n", VERSION, DATE);
