@@ -2,7 +2,7 @@
               library routine.
 
 %%% portions-copyright-cmetz-96
-Portions of this software are Copyright 1996-1998 by Craig Metz, All Rights
+Portions of this software are Copyright 1996-1999 by Craig Metz, All Rights
 Reserved. The Inner Net License Version 2 applies to these portions of
 the software.
 You should have received a copy of the license with this software. If
@@ -18,6 +18,8 @@ you didn't get a copy, you may request one from <license@inner.net>.
 */
 #include "opie_cfg.h"
 #include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
 #if DEBUG
 #include <syslog.h>
 #endif /* DEBUG */
