@@ -2,7 +2,7 @@
 opieinfo: Print a user's current OPIE sequence number and seed
 
 %%% portions-copyright-cmetz-96
-Portions of this software are Copyright 1996-1998 by Craig Metz, All Rights
+Portions of this software are Copyright 1996-1999 by Craig Metz, All Rights
 Reserved. The Inner Net License Version 2 applies to these portions of
 the software.
 You should have received a copy of the license with this software. If
@@ -33,6 +33,8 @@ License Agreement applies to this software.
 
 #include "opie_cfg.h"
 #include <stdio.h>
+#include <errno.h>
+#include <stdlib.h>
 #if HAVE_UNISTD_H
 #include <unistd.h>
 #endif /* HAVE_UNISTD_H */
@@ -42,7 +44,7 @@ License Agreement applies to this software.
 #include "opie.h"
 
 /* extern char *optarg; */
-extern int errno, optind;
+extern int optind;
 
 static char *getusername FUNCTION_NOARGS
 {
